@@ -9,8 +9,9 @@
 
 namespace PHP_CodeSniffer\Standards\Generic\Sniffs\Functions;
 
-use PHP_CodeSniffer\Sniffs\Sniff;
 use PHP_CodeSniffer\Files\File;
+use PHP_CodeSniffer\Sniffs\Sniff;
+use PHP_CodeSniffer\Util\Tokens;
 
 class OpeningFunctionBraceBsdAllmanSniff implements Sniff
 {
@@ -33,7 +34,7 @@ class OpeningFunctionBraceBsdAllmanSniff implements Sniff
     /**
      * Registers the tokens that this sniff wants to listen for.
      *
-     * @return void
+     * @return int[]
      */
     public function register()
     {
@@ -80,26 +81,61 @@ class OpeningFunctionBraceBsdAllmanSniff implements Sniff
             }
         }
 
-        $functionLine = $tokens[$closeBracket]['line'];
+        // Find the end of the function declaration.
+        $prev = $phpcsFile->findPrevious(Tokens::$emptyTokens, ($openingBrace - 1), $closeBracket, true);
+
+        $functionLine = $tokens[$prev]['line'];
         $braceLine    = $tokens[$openingBrace]['line'];
 
         $lineDifference = ($braceLine - $functionLine);
 
+        $metricType = 'Function';
+        if ($tokens[$stackPtr]['code'] === T_CLOSURE) {
+            $metricType = 'Closure';
+        }
+
         if ($lineDifference === 0) {
             $error = 'Opening brace should be on a new line';
             $fix   = $phpcsFile->addFixableError($error, $openingBrace, 'BraceOnSameLine');
             if ($fix === true) {
-                $phpcsFile->fixer->beginChangeset();
-                $indent = $phpcsFile->findFirstOnLine([], $openingBrace);
-                if ($tokens[$indent]['code'] === T_WHITESPACE) {
-                    $phpcsFile->fixer->addContentBefore($openingBrace, $tokens[$indent]['content']);
+                $hasTrailingAnnotation = false;
+                for ($nextLine = ($openingBrace + 1); $nextLine < $phpcsFile->numTokens; $nextLine++) {
+                    if ($tokens[$openingBrace]['line'] !== $tokens[$nextLine]['line']) {
+                        break;
+                    }
+
+                    if (isset(Tokens::$phpcsCommentTokens[$tokens[$nextLine]['code']]) === true) {
+                        $hasTrailingAnnotation = true;
+                    }
                 }
 
-                $phpcsFile->fixer->addNewlineBefore($openingBrace);
-                $phpcsFile->fixer->endChangeset();
-            }
+                $phpcsFile->fixer->beginChangeset();
+                $indent = $phpcsFile->findFirstOnLine([], $openingBrace);
 
-            $phpcsFile->recordMetric($stackPtr, 'Function opening brace placement', 'same line');
+                if ($hasTrailingAnnotation === false || $nextLine === false) {
+                    if ($tokens[$indent]['code'] === T_WHITESPACE) {
+                        $phpcsFile->fixer->addContentBefore($openingBrace, $tokens[$indent]['content']);
+                    }
+
+                    if ($tokens[($openingBrace - 1)]['code'] === T_WHITESPACE) {
+                        $phpcsFile->fixer->replaceToken(($openingBrace - 1), '');
+                    }
+
+                    $phpcsFile->fixer->addNewlineBefore($openingBrace);
+                } else {
+                    $phpcsFile->fixer->replaceToken($openingBrace, '');
+                    $phpcsFile->fixer->addNewlineBefore($nextLine);
+                    $phpcsFile->fixer->addContentBefore($nextLine, '{');
+
+                    if ($tokens[$indent]['code'] === T_WHITESPACE) {
+                        $phpcsFile->fixer->addContentBefore($nextLine, $tokens[$indent]['content']);
+                    }
+                }
+
+                $phpcsFile->fixer->endChangeset();
+            }//end if
+
+            $phpcsFile->recordMetric($stackPtr, "$metricType opening brace placement", 'same line');
         } else if ($lineDifference > 1) {
             $error = 'Opening brace should be on the line after the declaration; found %s blank line(s)';
             $data  = [($lineDifference - 1)];
@@ -116,7 +152,9 @@ class OpeningFunctionBraceBsdAllmanSniff implements Sniff
             }
         }//end if
 
-        $next = $phpcsFile->findNext(T_WHITESPACE, ($openingBrace + 1), null, true);
+        $ignore   = Tokens::$phpcsCommentTokens;
+        $ignore[] = T_WHITESPACE;
+        $next     = $phpcsFile->findNext($ignore, ($openingBrace + 1), null, true);
         if ($tokens[$next]['line'] === $tokens[$openingBrace]['line']) {
             if ($next === $tokens[$stackPtr]['scope_closer']) {
                 // Ignore empty functions.
@@ -167,7 +205,7 @@ class OpeningFunctionBraceBsdAllmanSniff implements Sniff
             }
         }//end if
 
-        $phpcsFile->recordMetric($stackPtr, 'Function opening brace placement', 'new line');
+        $phpcsFile->recordMetric($stackPtr, "$metricType opening brace placement", 'new line');
 
     }//end process()
 
