@@ -82,6 +82,10 @@ class ScopeIndentUnitTest extends AbstractSniffUnitTest
             ];
         }
 
+        if ($testFile === 'ScopeIndentUnitTest.4.inc') {
+            return [];
+        }
+
         return [
             7    => 1,
             10   => 1,
@@ -162,20 +166,22 @@ class ScopeIndentUnitTest extends AbstractSniffUnitTest
             1290 => 1,
             1292 => 1,
             1293 => 1,
-            1307 => 1,
-            1308 => 1,
-            1309 => 1,
             1310 => 1,
-            1311 => 1,
             1312 => 1,
-            1315 => 1,
-            1320 => 1,
-            1322 => 1,
-            1325 => 1,
-            1336 => 1,
-            1337 => 1,
-            1338 => 1,
-            1339 => 1,
+            1327 => 1,
+            1328 => 1,
+            1329 => 1,
+            1330 => 1,
+            1331 => 1,
+            1332 => 1,
+            1335 => 1,
+            1340 => 1,
+            1342 => 1,
+            1345 => 1,
+            1455 => 1,
+            1456 => 1,
+            1457 => 1,
+            1458 => 1,
         ];
 
     }//end getErrorList()
