@@ -9,8 +9,8 @@
 
 namespace PHP_CodeSniffer\Standards\PEAR\Sniffs\WhiteSpace;
 
-use PHP_CodeSniffer\Sniffs\Sniff;
 use PHP_CodeSniffer\Files\File;
+use PHP_CodeSniffer\Sniffs\Sniff;
 
 class ObjectOperatorIndentSniff implements Sniff
 {
@@ -22,6 +22,13 @@ class ObjectOperatorIndentSniff implements Sniff
      */
     public $indent = 4;
 
+    /**
+     * Indicates whether multilevel indenting is allowed.
+     *
+     * @var boolean
+     */
+    public $multilevel = false;
+
 
     /**
      * Returns an array of tokens this test wants to listen for.
@@ -71,12 +78,12 @@ class ObjectOperatorIndentSniff implements Sniff
             }
         }
 
-        $requiredIndent = 0;
+        $baseIndent = 0;
         if ($i >= 0 && $tokens[$i]['code'] === T_WHITESPACE) {
-            $requiredIndent = strlen($tokens[$i]['content']);
+            $baseIndent = $tokens[$i]['length'];
         }
 
-        $requiredIndent += $this->indent;
+        $baseIndent += $this->indent;
 
         // Determine the scope of the original object operator.
         $origBrackets = null;
@@ -96,6 +103,8 @@ class ObjectOperatorIndentSniff implements Sniff
             $next = $stackPtr;
         }
 
+        $previousIndent = $baseIndent;
+
         while ($next !== false) {
             // Make sure it is in the same scope, otherwise don't check indent.
             $brackets = null;
@@ -109,7 +118,7 @@ class ObjectOperatorIndentSniff implements Sniff
             }
 
             if ($origBrackets === $brackets && $origConditions === $conditions) {
-                // Make sure it starts a line, otherwise dont check indent.
+                // Make sure it starts a line, otherwise don't check indent.
                 $prev   = $phpcsFile->findPrevious(T_WHITESPACE, ($next - 1), $stackPtr, true);
                 $indent = $tokens[($next - 1)];
                 if ($tokens[$prev]['line'] !== $tokens[$next]['line']
@@ -121,16 +130,26 @@ class ObjectOperatorIndentSniff implements Sniff
                         $foundIndent = 0;
                     }
 
-                    if ($foundIndent !== $requiredIndent) {
+                    $minIndent      = $previousIndent;
+                    $maxIndent      = $previousIndent;
+                    $expectedIndent = $previousIndent;
+
+                    if ($this->multilevel === true) {
+                        $minIndent      = max(($previousIndent - $this->indent), $baseIndent);
+                        $maxIndent      = ($previousIndent + $this->indent);
+                        $expectedIndent = min(max($foundIndent, $minIndent), $maxIndent);
+                    }
+
+                    if ($foundIndent < $minIndent || $foundIndent > $maxIndent) {
                         $error = 'Object operator not indented correctly; expected %s spaces but found %s';
                         $data  = [
-                            $requiredIndent,
+                            $expectedIndent,
                             $foundIndent,
                         ];
 
                         $fix = $phpcsFile->addFixableError($error, $next, 'Incorrect', $data);
                         if ($fix === true) {
-                            $spaces = str_repeat(' ', $requiredIndent);
+                            $spaces = str_repeat(' ', $expectedIndent);
                             if ($foundIndent === 0) {
                                 $phpcsFile->fixer->addContentBefore($next, $spaces);
                             } else {
@@ -138,6 +157,8 @@ class ObjectOperatorIndentSniff implements Sniff
                             }
                         }
                     }
+
+                    $previousIndent = $expectedIndent;
                 }//end if
 
                 // It cant be the last thing on the line either.
