@@ -9,55 +9,11 @@
 
 namespace PHP_CodeSniffer\Tests\Core\File;
 
-use PHP_CodeSniffer\Config;
-use PHP_CodeSniffer\Ruleset;
-use PHP_CodeSniffer\Files\DummyFile;
-use PHPUnit\Framework\TestCase;
+use PHP_CodeSniffer\Tests\Core\AbstractMethodUnitTest;
 
-class GetMethodParametersTest extends TestCase
+class GetMethodParametersTest extends AbstractMethodUnitTest
 {
 
-    /**
-     * The PHP_CodeSniffer_File object containing parsed contents of the test case file.
-     *
-     * @var \PHP_CodeSniffer\Files\File
-     */
-    private $phpcsFile;
-
-
-    /**
-     * Initialize & tokenize PHP_CodeSniffer_File with code from the test case file.
-     *
-     * Methods used for these tests can be found in a test case file in the same
-     * directory and with the same name, using the .inc extension.
-     *
-     * @return void
-     */
-    public function setUp()
-    {
-        $config            = new Config();
-        $config->standards = ['Generic'];
-
-        $ruleset = new Ruleset($config);
-
-        $pathToTestFile  = dirname(__FILE__).'/'.basename(__FILE__, '.php').'.inc';
-        $this->phpcsFile = new DummyFile(file_get_contents($pathToTestFile), $ruleset, $config);
-        $this->phpcsFile->process();
-
-    }//end setUp()
-
-
-    /**
-     * Clean up after finished test.
-     *
-     * @return void
-     */
-    public function tearDown()
-    {
-        unset($this->phpcsFile);
-
-    }//end tearDown()
-
 
     /**
      * Verify pass-by-reference parsing.
@@ -76,18 +32,7 @@ class GetMethodParametersTest extends TestCase
             'nullable_type'     => false,
         ];
 
-        $start    = ($this->phpcsFile->numTokens - 1);
-        $function = $this->phpcsFile->findPrevious(
-            T_COMMENT,
-            $start,
-            null,
-            false,
-            '/* testPassByReference */'
-        );
-
-        $found = $this->phpcsFile->getMethodParameters(($function + 2));
-        unset($found[0]['token']);
-        $this->assertSame($expected, $found);
+        $this->getMethodParametersTestHelper('/* '.__FUNCTION__.' */', $expected);
 
     }//end testPassByReference()
 
@@ -109,18 +54,7 @@ class GetMethodParametersTest extends TestCase
             'nullable_type'     => false,
         ];
 
-        $start    = ($this->phpcsFile->numTokens - 1);
-        $function = $this->phpcsFile->findPrevious(
-            T_COMMENT,
-            $start,
-            null,
-            false,
-            '/* testArrayHint */'
-        );
-
-        $found = $this->phpcsFile->getMethodParameters(($function + 2));
-        unset($found[0]['token']);
-        $this->assertSame($expected, $found);
+        $this->getMethodParametersTestHelper('/* '.__FUNCTION__.' */', $expected);
 
     }//end testArrayHint()
 
@@ -151,19 +85,7 @@ class GetMethodParametersTest extends TestCase
             'nullable_type'     => false,
         ];
 
-        $start    = ($this->phpcsFile->numTokens - 1);
-        $function = $this->phpcsFile->findPrevious(
-            T_COMMENT,
-            $start,
-            null,
-            false,
-            '/* testTypeHint */'
-        );
-
-        $found = $this->phpcsFile->getMethodParameters(($function + 2));
-        unset($found[0]['token']);
-        unset($found[1]['token']);
-        $this->assertSame($expected, $found);
+        $this->getMethodParametersTestHelper('/* '.__FUNCTION__.' */', $expected);
 
     }//end testTypeHint()
 
@@ -185,18 +107,7 @@ class GetMethodParametersTest extends TestCase
             'nullable_type'     => false,
         ];
 
-        $start    = ($this->phpcsFile->numTokens - 1);
-        $function = $this->phpcsFile->findPrevious(
-            T_COMMENT,
-            $start,
-            null,
-            false,
-            '/* testSelfTypeHint */'
-        );
-
-        $found = $this->phpcsFile->getMethodParameters(($function + 2));
-        unset($found[0]['token']);
-        $this->assertSame($expected, $found);
+        $this->getMethodParametersTestHelper('/* '.__FUNCTION__.' */', $expected);
 
     }//end testSelfTypeHint()
 
@@ -227,19 +138,7 @@ class GetMethodParametersTest extends TestCase
             'nullable_type'     => true,
         ];
 
-        $start    = ($this->phpcsFile->numTokens - 1);
-        $function = $this->phpcsFile->findPrevious(
-            T_COMMENT,
-            $start,
-            null,
-            false,
-            '/* testNullableTypeHint */'
-        );
-
-        $found = $this->phpcsFile->getMethodParameters(($function + 2));
-        unset($found[0]['token']);
-        unset($found[1]['token']);
-        $this->assertSame($expected, $found);
+        $this->getMethodParametersTestHelper('/* '.__FUNCTION__.' */', $expected);
 
     }//end testNullableTypeHint()
 
@@ -261,18 +160,7 @@ class GetMethodParametersTest extends TestCase
             'nullable_type'     => false,
         ];
 
-        $start    = ($this->phpcsFile->numTokens - 1);
-        $function = $this->phpcsFile->findPrevious(
-            T_COMMENT,
-            $start,
-            null,
-            false,
-            '/* testVariable */'
-        );
-
-        $found = $this->phpcsFile->getMethodParameters(($function + 2));
-        unset($found[0]['token']);
-        $this->assertSame($expected, $found);
+        $this->getMethodParametersTestHelper('/* '.__FUNCTION__.' */', $expected);
 
     }//end testVariable()
 
@@ -295,18 +183,7 @@ class GetMethodParametersTest extends TestCase
             'nullable_type'     => false,
         ];
 
-        $start    = ($this->phpcsFile->numTokens - 1);
-        $function = $this->phpcsFile->findPrevious(
-            T_COMMENT,
-            $start,
-            null,
-            false,
-            '/* testSingleDefaultValue */'
-        );
-
-        $found = $this->phpcsFile->getMethodParameters(($function + 2));
-        unset($found[0]['token']);
-        $this->assertSame($expected, $found);
+        $this->getMethodParametersTestHelper('/* '.__FUNCTION__.' */', $expected);
 
     }//end testSingleDefaultValue()
 
@@ -338,19 +215,7 @@ class GetMethodParametersTest extends TestCase
             'nullable_type'     => false,
         ];
 
-        $start    = ($this->phpcsFile->numTokens - 1);
-        $function = $this->phpcsFile->findPrevious(
-            T_COMMENT,
-            $start,
-            null,
-            false,
-            '/* testDefaultValues */'
-        );
-
-        $found = $this->phpcsFile->getMethodParameters(($function + 2));
-        unset($found[0]['token']);
-        unset($found[1]['token']);
-        $this->assertSame($expected, $found);
+        $this->getMethodParametersTestHelper('/* '.__FUNCTION__.' */', $expected);
 
     }//end testDefaultValues()
 
@@ -373,20 +238,58 @@ class GetMethodParametersTest extends TestCase
             'nullable_type'     => false,
         ];
 
-        $start    = ($this->phpcsFile->numTokens - 1);
-        $function = $this->phpcsFile->findPrevious(
-            T_COMMENT,
-            $start,
-            null,
-            false,
-            '/* testBitwiseAndConstantExpressionDefaultValue */'
-        );
-
-        $found = $this->phpcsFile->getMethodParameters(($function + 2));
-        unset($found[0]['token']);
-        $this->assertSame($expected, $found);
+        $this->getMethodParametersTestHelper('/* '.__FUNCTION__.' */', $expected);
 
     }//end testBitwiseAndConstantExpressionDefaultValue()
 
 
+    /**
+     * Verify that arrow functions are supported.
+     *
+     * @return void
+     */
+    public function testArrowFunction()
+    {
+        $expected    = [];
+        $expected[0] = [
+            'name'              => '$a',
+            'content'           => 'int $a',
+            'pass_by_reference' => false,
+            'variable_length'   => false,
+            'type_hint'         => 'int',
+            'nullable_type'     => false,
+        ];
+
+        $expected[1] = [
+            'name'              => '$b',
+            'content'           => '...$b',
+            'pass_by_reference' => false,
+            'variable_length'   => true,
+            'type_hint'         => '',
+            'nullable_type'     => false,
+        ];
+
+        $this->getMethodParametersTestHelper('/* '.__FUNCTION__.' */', $expected);
+
+    }//end testArrowFunction()
+
+
+    /**
+     * Test helper.
+     *
+     * @param string $commentString The comment which preceeds the test.
+     * @param array  $expected      The expected function output.
+     *
+     * @return void
+     */
+    private function getMethodParametersTestHelper($commentString, $expected)
+    {
+        $function = $this->getTargetToken($commentString, [T_FUNCTION, T_FN]);
+        $found    = self::$phpcsFile->getMethodParameters($function);
+
+        $this->assertArraySubset($expected, $found, true);
+
+    }//end getMethodParametersTestHelper()
+
+
 }//end class
